from pprint import pprint
from datetime import datetime, timedelta

from pydantic import BaseModel


# def test_get_students(hh_client, test_data):
#     response = hh_client.clients.get_students(term='9226036452', by_agents=False).students
#     response1 = hh_client.ed_unit_students.get_students(student_client_id=response[-1].client_id, query_days=True)
#     res = list(filter(lambda group: group.end_date != None and group.end_date >= datetime.now() and group.ed_unit_discipline != 'Отработка', response1))
#     res1 = hh_client.ed_unit_students.get_students(student_client_id=response[-1].client_id, ed_unit_id=res[-1].ed_unit_id, query_days=True, date_from=datetime.strptime('01-09-2023', '%d-%m-%Y'), date_to=res[-1].end_date)[-1]
    
#     days = res1.days
#     # print(datetime.strptime('01-09-2023', '%d-%m-%Y'), res[-1].end_date)
#     days = list(filter(lambda day: day.date <= datetime.now() and day.date == datetime.strptime('2024-01-13', '%Y-%m-%d'), days))
#     pprint(days)
#     # # # eds = hh_client.ed_units.get_ed_units(id=res[-1].ed_unit_id, query_days=True)
#     # # # print(days[-1].date.year)
#     # print(res1.ed_unit_name, res1.ed_unit_office_or_company_name, response[-1].client_id )
#     # pprint(days[-1].date.strftime('%m–%d–%Y'))


def test_get_working_off_days(hh_client):
    response = hh_client.ed_units.get_ed_units(statuses='Working', location_id=1, disciplines='Отработка', query_days=True )
    workoff_days = []
    for _g in response:
        days = _g.days
        days = list(filter(lambda day: day.date >= datetime.now(), days))
        days = [day.date.strftime('%d-%m-%Y') for day in days]
        workoff_days.append({'name': _g.name, 'days': days, 'ed_unit _id': _g.id, 'office_name': _g.office_or_company_name, 'office_id': _g.office_or_company_id})
    # pprint(workoff_days)




key_not_study_description = ['Ещё не начал заниматься', 'Перенос на другой день']

class WorkOffsResponse(BaseModel):
    name: str 
    days: list[str]
    ed_unit_id: int
    office_name: str
    office_id: int

class PassDaysResponse(BaseModel):
    group_name: str
    groups_of_passes: list[list]


def filter_normal_day(day_obj):
    if not day_obj.description is None:  
        for key in key_not_study_description:
            if key in day_obj.description:
                return False
    if day_obj.date <= datetime.now():
        return True
    else: 
        return False


def test_get_passes(hh_client):
    
    response = hh_client.clients.get_students(term='9226036452', by_agents=False).students[-1]
    print(response.client_id)
    response = hh_client.ed_unit_students.get_students(student_client_id=response.client_id, query_days=True)
    working_groups = list(filter(lambda group: (group.end_date != None) and group.end_date >= datetime.now() and group.ed_unit_discipline != 'Отработка', response))
    
    
    passes_objs = []
    for group in working_groups:
        group_with_days = hh_client.ed_unit_students.get_students(
            student_client_id=group.student_client_id, 
            ed_unit_id=group.ed_unit_id, 
            query_days=True, 
            date_from=group.begin_date, 
            date_to=group.end_date
            )[-1]
        days = group_with_days.days

        # filter by days that the student actually missed or was in, like a paid day 
        days = list(filter(filter_normal_day, days))
        # filter by days on which the student was not present in class 
        missed_days = list(filter(lambda day: day.pass_, days))

        couple_of_passes_day = []
        paired_passes_day = []

        for index, day in enumerate(missed_days):
            if (index + 1) % 2:
                couple_of_passes_day.append(day)
            else:
                couple_of_passes_day.append(day)
                paired_passes_day.append(couple_of_passes_day)
                couple_of_passes_day = []
        else:
            if couple_of_passes_day:
                paired_passes_day.append(couple_of_passes_day)
                
        finally_paired_passes_day = []

        for couple_of_passes in paired_passes_day:
            if couple_of_passes[-1].working_off_to_ed_unit_id is None:
                finally_paired_passes_day.append(couple_of_passes)
        
        paired_passes_day_str = []
        for couple_of_passes in finally_paired_passes_day:
            _pair_str = []
            for pass_day in couple_of_passes:
                _pair_str.append(pass_day.date.strftime('%d-%m-%Y'))
            paired_passes_day_str.append(_pair_str)


        passes_objs.append(PassDaysResponse(group_name=group_with_days.ed_unit_name, groups_of_passes=paired_passes_day_str))
    pprint(passes_objs)


from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, NoReturn

if TYPE_CHECKING:
    from requests import Response

from hollihop_api_client.categories import APICategories


class AbstractAPI(ABC, APICategories):
    @abstractmethod
    def request(
            self,
            method: str,
            http_method: str = 'GET',
            data: None | dict = None,
    ) -> None | dict:
        pass

    @abstractmethod
    def json_request(
        self,
        method,
        http_method: str = 'GET',
        data: None | dict = None,
    ) -> None | dict:
        pass

    @abstractmethod
    def _validate_response(
            self,
            response: "Response"
    ) -> (None | dict) | NoReturn:
        pass


__all__ = ['AbstractAPI']



from datetime import datetime

def test_get_students(hh_client, test_data):
    response = hh_client.clients.get_students(term='9226036452', by_agents=False).students
    client_id = response[0].client_id
    response = hh_client.ed_unit_students.get_students(student_client_id=client_id, query_days=True)
    response = list(filter(lambda group: group.end_date != None and group.end_date >= datetime.now() and group.ed_unit_discipline != 'Отработка', response))[-1]
    ed_unit_id = response.ed_unit_id
    response = hh_client.ed_units.set_student_passes(
        data=[
                hh_client.ed_units.create_student_pass_data(
                date='2024-01-13', 
                ed_unit_id=ed_unit_id, 
                student_client_id=client_id, 
                pass_=True, overwriteAcceptedManually=True, 
                acceptedDescription='Отработка в 1.4 ОТР 27.01',
                description='test desc 1'
            ),
                hh_client.ed_units.create_student_pass_data(
                date='2024-01-20', 
                ed_unit_id=ed_unit_id, 
                student_client_id=client_id, 
                pass_=True, overwriteAcceptedManually=True, 
                acceptedDescription='Отработка в 1.4 ОТР 27.01',
                description='test desc 2'
            )
        ]
        )
    print(response)


import json


def test_get_lead(hh_client, test_data):
    response = hh_client.leads.get_leads(id=test_data.test_lead_id)
    print(response[-1].agents)
    assert response[-1].name == test_data.test_lead_name


def test_add_lead(hh_client):
    response = hh_client.leads.add_lead(
        first_name='---',
        last_name='---',
        birthday='2023-01-01',
        office_or_company_id=3,
        status='Звонобот',
        ad_source='Звонобот',
        comment='Тестовый лид со Звонобота',
    )
    resp = hh_client.leads.edit_contacts(
        lead_id=response.lead_id,
        mobile='79226036452',
        email='clients@algoritmika.org'
    )
    print(response)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


class BaseCategory:
    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    @classmethod
    def handle_parameters(cls, parameters: dict) -> dict:
        handled_parameters = parameters.copy()

        handled_parameters.pop('self')

        for key, value in parameters.items():
            if value is None:
                handled_parameters.pop(key)
        return handled_parameters


__all__ = ['BaseCategory']



import logging
from functools import lru_cache
from typing import Any

import phonenumbers

keyword_tuple = ('False', 'await', 'else', 'import', 'pass',
                 'None', 'break', 'except', 'in', 'raise',
                 'True', 'class', 'finally', 'is', 'return',
                 'and', 'continue', 'for', 'lambda', 'try',
                 'as', 'def', 'from', 'nonlocal', 'while',
                 'assert', 'del', 'global', 'not', 'with',
                 'async', 'elif', 'if', 'or', 'yield',
                 )


def dict_to_camel(data: dict[Any, Any]) -> dict[Any, Any]:
    converted: dict[Any, Any] = {}
    for k, v in data.items():
        if isinstance(k, str):
            key = to_camel(k)
        else:
            key = k

        if isinstance(v, dict):
            converted[key] = dict_to_camel(v)
        elif isinstance(v, list):
            converted[key] = [dict_to_camel(
                x) if isinstance(x, dict) else x for x in v]
        elif isinstance(v, tuple):
            converted[key] = tuple(dict_to_camel(
                x) if isinstance(x, dict) else x for x in v)
        else:
            converted[key] = data[k]

    return converted


def dict_to_snake(data: dict[Any, Any]) -> dict[Any, Any]:
    converted: dict[Any, Any] = {}
    for k, v in data.items():
        if isinstance(k, str):
            if k.lower() in keyword_tuple:
                key = k.capitalize()
            elif 'EMail' in k:
                key = to_snake(k.replace('EMail', 'Email'))
            else:
                key = to_snake(k)
        else:
            key = k

        if isinstance(v, dict):
            converted[key] = dict_to_snake(v)
        elif isinstance(v, list):
            converted[key] = [dict_to_snake(
                x) if isinstance(x, dict) else x for x in v]
        elif isinstance(v, tuple):
            converted[key] = tuple(dict_to_snake(
                x) if isinstance(x, dict) else x for x in v)
        else:
            converted[key] = data[k]

    return converted


def dict_to_pascal(data: dict[Any, Any]) -> dict[Any, Any]:
    converted: dict[Any, Any] = {}
    for k, v in data.items():
        if isinstance(k, str):
            key = to_pascal(k)
        else:
            key = k

        if isinstance(v, dict):
            converted[key] = dict_to_pascal(v)
        elif isinstance(v, list):
            converted[key] = [dict_to_pascal(
                x) if isinstance(x, dict) else x for x in v]
        elif isinstance(v, tuple):
            converted[key] = tuple(dict_to_pascal(
                x) if isinstance(x, dict) else x for x in v)
        else:
            converted[key] = data[k]

    return converted


@lru_cache(maxsize=4096)
def to_camel(snake_string: str) -> str:
    words = _split_snake(snake_string)

    return words[0] + "".join(word.title() for word in words[1:])


@lru_cache(maxsize=4096)
def to_snake(camel_string: str) -> str:
    return "".join([f"_{c}" if c.isupper() else c for c in camel_string]).lstrip("_").lower()


@lru_cache(maxsize=4096)
def to_pascal(snake_string: str) -> str:
    words = _split_snake(snake_string)

    return "".join(word.title() for word in words)


def _split_snake(snake_string: str) -> list[str]:
    return snake_string.split("_")


def format_phone(phone: str) -> str:
    try:
        return phonenumbers.format_number(
            phonenumbers.parse(phone, 'RU'),
            phonenumbers.PhoneNumberFormat.E164
        )
    except Exception as error:
        logging.error(
            'Number is not formatted\nError – {error}'.format(error=error))
        return phone



from datetime import date

from pydantic import BaseModel


class GroupRequest(BaseModel):
    id: None | str = None,

    date_from: None | str = None,
    date_to: None | str = None,

    learning_types: None | list[str] = None,
    levels: None | list[str] = None,
    disciplines: None | list[str] = None,

    statuses: None | str = None,
    office_or_company_id: None | str = None,
    location_id: None | list[str] = None,
    maturities: None | str = None,
    corporative: None | bool = None,
    types: None | str = None,
    teacher_id: None | str = None,
    query_days: None | bool = None,
    query_fiscal_info: None | bool = None,
    query_teacher_prices: None | bool = None


class StudentRequest(BaseModel):
    ed_unit_id: None | int = None,
    ed_unit_types: None | str = None,
    ed_unit_office_or_company_id: None | int = None,
    ed_unit_office_or_company: None | str = None,
    student_client_id: None | int = None,
    date_from: None | date = None,
    date_to: None | date = None,
    statuses: None | str = None,
    query_payers: None | bool = None,
    query_days: None | bool = None



from datetime import datetime
from dataclasses import dataclass, asdict


@dataclass
class SetStudentPasses: 
    date: str | None = None
    ed_unit_id: int | None = None
    student_client_id: int | None = None
    pass_: bool | None = None
    payable: bool | None = None
    description: str | None = None
    acceptedDescription: str | None = None
    overwriteAcceptedManually: bool | None = None


def test_get_students(hh_client, test_data):
    response = hh_client.clients.get_students(term='9226036452', by_agents=False).students
    client_id = response[-1].client_id
    response = hh_client.ed_units.add_ed_unit_student(
        ed_unit_id=711,
        student_client_id=client_id,
        status='WorkingOff',
        begin='2023-09-23',
        end='2024-01-27'
    )
    print(response)
    response = hh_client.ed_units.set_student_passes(
        data=[
            asdict(SetStudentPasses(date='2024-01-27', ed_unit_id=711, student_client_id=client_id, pass_=True, overwriteAcceptedManually=True, acceptedDescription='Отработка 1.1 СВС 13.01')),
            asdict(SetStudentPasses(date='2024-01-13', ed_unit_id=711, student_client_id=client_id, pass_=True, overwriteAcceptedManually=True, acceptedDescription='Отработка 1.1 СВС 20.01'))
            ]
        )
    print(response)



from dataclasses import dataclass
from datetime import datetime
from typing import TYPE_CHECKING

from hollihop_api_client.base.category import BaseCategory
from hollihop_api_client.tools import dict_to_camel, dict_to_snake


if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


class Phone(str):
    def __repr__(self):
        return self.replace("'", "", 2)

    def __str__(self):
        return f"{self[:2]}xxxxxx{self[-4:]}"

@dataclass
class UTM:
     source: str | None = None
     medium: str | None = None
     campaign: str | None = None
     term: str | None = None
     content: str | None = None

@dataclass
class StudyRequest:
    id: int | None = None
    created: datetime | None = None
    utm: list[UTM] | None = None
    reffer: str | None = None
    lead_id: int | None = None
    
@dataclass
class ExtraField:
    name: str | None = None
    value: str | None = None
    
@dataclass
class Assignee:
    id: int | None = None
    full_name: str | None = None

@dataclass
class OfficesAndCompany:
    id: int | None = None
    name: str | None = None
     
    
@dataclass
class Discipline:
    discipline: str | None = None
    level: str | None = None


@dataclass
class Agent:
    first_name: None | str = None
    last_name: None | str = None
    middle_name: None | str = None
    who_is: None | str = None
    mobile: None | str = None
    use_mobile_by_system: None | str = None
    phone: None | Phone = None
    email: None | str = None
    use_email_by_system: None | bool = None
    skype: None | str = None
    job_or_study_place: None | str = None
    position: None | str = None
    is_customer: None | bool = None
    birthday: None | datetime = None


@dataclass
class Student:
    client_id: int | None = None
    id: int | None = None
    created: datetime | None = None
    updated: datetime | None = None
    first_name: str | None = None
    last_name: str | None = None
    middle_name: str | None = None
    photo_urls: str | None = None
    gender: str | None = None
    address_date: datetime | None = None
    ad_soure: str | None = None
    visit_date_time: datetime | None = None
    status_id: int | None = None
    status: str | None = None
    birthday: datetime | None = None
    mobile: str | None = None
    use_mobile_by_system: bool | None = None
    email: str | None = None
    use_email_by_system: bool | None = None
    skype: str | None = None
    address: str | None = None
    social_network_page: str | None = None
    job_or_study_place: str | None = None
    position: str | None = None
    agents: list[Agent] | None = None
    maturity: str | None = None
    learning_types: list[str] | None = None
    disciplines: list[Discipline] | None = None
    offices_and_companies: list[OfficesAndCompany] | None = None
    assignees: list[Assignee] | None = None
    extra_fields: list[ExtraField] | None = None
    ad_source: str | None = None
    phone: str | None = None


@dataclass
class Students:
    students: list[Student] | None = None
    now: datetime | None = None

    def __post_init__(self):
         if self.students: 
              self.students = [Student(**_) for _ in self.students]

class Clients(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def get_students(
            self,
            client_id: int | None = None, 
            id: int | None = None, 
            ed_unit_office_or_company_id: int | None = None,
            term: str | None = None,
            by_agents: bool | None = False 
    ) -> Students:
        data = dict_to_camel(self.handle_parameters(locals()))
        response = self.api.request(
            method='GetStudents',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)

        return Students(**response)
        


def test_get_locations_by_id(hh_client, test_data):
    response = hh_client.locations.get_locations(id=test_data.test_city_id)
    print(response)
    assert response[-1].name == test_data.test_city_name


def test_get_locations_by_name(hh_client, test_data):
    response = hh_client.locations.get_locations(name=test_data.test_city_name)
    assert response[-1].id == test_data.test_city_id



from dataclasses import dataclass, field
from datetime import datetime
from typing import TYPE_CHECKING, Any

from hollihop_api_client.base import BaseCategory
from hollihop_api_client.tools import (dict_to_camel, dict_to_snake,
                                       format_phone)

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


@dataclass
class Agent:
    first_name: str | None = None
    last_name: str | None = None
    middle_name: str | None = None
    is_customer: bool | None = None
    who_is: str | None = None
    email: str | None = None
    use_email_by_system: bool | None = None
    phone: str | None = None
    mobile: str | None = None
    use_mobile_by_system: bool | None = None
    job_or_study_place: str | None = None
    position: str | None = None
    birthday: datetime | None = None
    skype: str | None = None

    def __post_init__(self):
        if not self.phone is None:
            self.phone = format_phone(self.phone.replace('+', ''))


@dataclass
class ExtraField:
    name: str | None = None
    value: str | None = None


@dataclass
class Lead:
    id: int | None
    created: datetime | None = None
    updated: datetime | None = None
    first_name: str | None = None
    last_name: str | None = None
    middle_name: str | None = None
    address_date: datetime | None = None
    ad_source: str | None = None
    status_id: str | None = None
    status: str | None = None
    birthday: datetime | None = None
    phone: str | None = None
    mobile: str | None = None
    use_mobile_by_system: bool | None = None
    email: str | None = None
    use_email_by_system: bool | None = None
    maturity: str | None = None
    learning_type: str | None = None
    discipline: str | None = None
    level: str | None = None
    agents: list[Agent] | None = None
    offices_and_companies: list | None = None
    assignees: list | None = None
    extra_fields: list | None = None
    student_client_id: int | None = None
    name: str = field(init=False)

    def __post_init__(self):
        self.name = ' '.join(filter(lambda str_obj: str_obj, [
                             self.last_name, self.first_name, self.middle_name]))
        if self.agents:
            self.agents = [Agent(**_) for _ in self.agents]
        if self.extra_fields:
            self.extra_fields = [ExtraField(**_) for _ in self.extra_fields]
        if not self.mobile is None:
            self.mobile = format_phone(self.mobile.replace('+', ''))
        if not self.phone is None:
            self.phone = format_phone(self.phone.replace('+', ''))


@dataclass
class Leads:
    leads: None | list[Lead] = field(default_factory=list)
    now: None | datetime = None


@dataclass
class AddLeadResponse:
    lead_id: int | None = None


@dataclass
class AddLeadsResponse:
    leads: list[AddLeadResponse] | None = None


class LeadsCategory(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def add_lead(
            self,
            first_name: str | None = None,
            middle_name: str | None = None,
            last_name: str | None = None,
            gender: bool | None = True,
            birthday: datetime | None = None,
            status: str | None = None,
            office_or_company_id: int | None = None,
            comment: str | None = None,
            ad_source: str | None = None,
    ):
        data = dict_to_camel(self.handle_parameters(locals()))
        response = self.api.request(
            method='AddLead',
            http_method='POST',
            data=data
        )

        lead = AddLeadResponse(**dict_to_snake(response))

        return lead

    def edit_contacts(
            self,
            lead_id: int | None = None,
            mobile: str | None = None,
            use_mobile_by_system: bool | None = True,
            use_email_by_system: bool | None = True,
            email: str | None = None
    ):
        data = dict_to_camel(self.handle_parameters(locals()))
        
        response = self.api.request(
            method='EditContacts',
            http_method='POST',
            data=data
        )

        return response

    def get_leads(
            self,
            id: None | int = None,
            attached: None | bool = None,
            student_client_id: None | int = None,
            office_or_company_id: int | None = None,
            created_from: datetime | None = None
    ) -> list[Lead]:
        data = dict_to_camel(self.handle_parameters(locals()))

        response = self.api.request(
            method='GetLeads',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)

        raw_leads = [Lead(**_) for _ in Leads(**response).leads]

        return raw_leads

    def get_all_leads_id(self, **kwargs) -> list[int]:
        leads = self.get_leads(**kwargs)
        return [lead.student_client_id for lead in leads]


__all__ = ['LeadsCategory']



import logging
from dataclasses import dataclass, field
from datetime import datetime, time
from typing import TYPE_CHECKING

import phonenumbers

from hollihop_api_client.base import BaseCategory
from hollihop_api_client.tools import dict_to_camel, dict_to_snake

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


def replace_spaces(from_str: str):
    return from_str.replace('\xa0', ' ')


class Phone(str):
    def __repr__(self):
        return self.replace("'", "", 2)

    def __str__(self):
        return f"{self[:2]}xxxxxx{self[-4:]}"
    
@dataclass
class Day:
    date: None | datetime = None
    minutes: None | float = None
    pass_: None | bool = field(init=False)
    Pass: None | bool = None
    student_payable_minutes: None | float = None
    teacher_payable_minutes: None | float = None
    description: str | None = None
    accepted: str | None = None
    accepted_description: str | None = None
    working_off_to_ed_unit_id: int | None = None
    working_off_to_date: datetime | None = None
    working_off_from_ed_unit_id: int | None = None
    working_off_from_date: datetime | None = None


    def __post_init__(self):
        if not self.date is None:
            self.date = datetime.fromisoformat(self.date)
        self.pass_ = self.Pass
        del self.Pass


@dataclass
class StudentAgent:
    first_name: None | str = None
    last_name: None | str = None
    middle_name: None | str = None
    who_is: None | str = None
    mobile: None | str = None
    use_mobile_by_system: None | str = None
    phone: None | Phone = None
    email: None | str = None
    use_email_by_system: None | bool = None
    skype: None | str = None
    job_or_study_place: None | str = None
    position: None | str = None
    is_customer: None | bool = None
    birthday: None | datetime = None


@dataclass
class Payer:
    client_id: int | None = None
    is_company: bool | None = None
    name: str | None = None
    actual: bool | None = None
    terminated_contracts: list | None = None
    price_id: int | None = None
    price_name: str | None = None
    discounts: list | None = None
    surcharges: list | None = None
    payable_minutes: int | None = None
    payable_units: str | None = None
    payable_minutes_ranged: int | None = None
    payable_units_ranged: str | None = None
    value: str | None = None
    value_ranged: str | None = None
    value_paid_ranged: str | None = None
    contract_value: str | None = None
    contract_value_restored: str | None = None
    contract_value_ranged: str | None = None
    contract_value_ranged_restored: str | None = None
    debt_date: datetime | None = field(init=True, default=None)
    ed_unit_payments: list | None = None

    def __post_init__(self):
        if self.debt_date:
            self.debt_date = datetime.fromisoformat(self.debt_date)
        if self.value:
            self.value = replace_spaces(self.value)
        if self.contract_value:
            self.contract_value = replace_spaces(self.contract_value)
        if self.contract_value_restored:
            self.contract_value_restored = replace_spaces(
                self.contract_value_restored)
        if self.price_name:
            self.price_name = replace_spaces(self.price_name)


def format_phone(number: str) -> Phone:
    if (number[:2]) == "+8":
        logging.warning(f"Неправильный формат номера: {number}")
        number = number[1:]
    return Phone(phonenumbers.format_number(
        phonenumbers.parse(number, 'RU'),
        phonenumbers.PhoneNumberFormat.E164).replace("+", ""))


@dataclass
class Student:
    ed_unit_id: int | None = None
    ed_unit_type: str | None = None
    ed_unit_name: str | None = None
    ed_unit_corporative: bool | None = None
    ed_unit_office_or_company_id: int | None = None
    ed_unit_office_or_company_name: str | None = None
    ed_unit_discipline: str | None = None
    ed_unit_level: str | None = None
    ed_unit_maturity: str | None = None
    ed_unit_learning_type: str | None = None
    student_client_id: int | None = None
    student_name: str | None = None
    student_mobile: str | None = None
    student_phone: str | None = None
    student_email: str | None = None
    student_agents: list[StudentAgent] | None = None
    student_extra_fields: list | None = None
    begin_date: datetime | None = None
    end_date: datetime | None = None
    begin_time: time | None = None
    end_time: time | None = None
    weekdays: int | None = None
    status: str | None = None
    study_minutes: int | None = None
    study_units: str | None = None
    days: list[Day] | None = None
    payers: list[Payer] | None = None
    phones: list[Phone] = field(init=False)

    def __post_init__(self):
        self.phones = []
        if self.payers:
            self.payers = [Payer(**_) for _ in self.payers]
        if self.student_agents:
            self.student_agents = [StudentAgent(
                **_) for _ in self.student_agents]
            for agent in self.student_agents:
                if agent.mobile:
                    self.phones.append(format_phone(
                        agent.mobile))
        if self.student_mobile:
            self.phones.append(format_phone(
                self.student_mobile))
        if self.student_phone:
            self.phones.append(format_phone(
                self.student_phone))
        if self.begin_date:
            self.begin_date = datetime.fromisoformat(self.begin_date)
        if self.end_date:
            self.end_date = datetime.fromisoformat(self.end_date)
        if not self.days is None:
            self.days = [Day(**_) for _ in self.days]

    # def __repr__(self) -> str:
    #     return self.student_name


@dataclass
class Students:
    ed_unit_students: list[Student] = field(default_factory=list)


class StudentsCategory(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def get_students(
            self,
            ed_unit_id: None | int = None,
            ed_unit_types: None | str = None,
            ed_unit_office_or_company_id: None | int = None,
            ed_unit_office_or_company: None | str = None,
            student_client_id: None | int = None,
            date_from: None | datetime = None,
            date_to: None | datetime = None,
            statuses: None | str = None,
            query_payers: None | bool = None,
            query_days: None | bool = None
    ) -> list[Student]:
        data = dict_to_camel(self.handle_parameters(locals()))

        response = self.api.request(
            method='GetEdUnitStudents',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)

        return [Student(**_) for _ in Students(**response).ed_unit_students]

    def get_all_students_name(self, **kwargs) -> list[str]:
        students = self.get_students(**kwargs)
        return [student.student_name for student in students]

    def get_all_students_id(self, **kwargs) -> list[int]:
        students = self.get_students(**kwargs)
        return [student.student_client_id for student in students]

    def get_students_with_debt(
            self,
            ed_unit_id: int,
            date_from: datetime | None = None,
            date_to: datetime | None = None,
            **kwargs):
        students = self.get_students(
            date_from=date_from,
            date_to=date_to,
            ed_unit_id=ed_unit_id,
            query_payers=True,
            **kwargs
        )

        students = filter(
            lambda student: student.payers[-1].debt_date != None, students
        )

        if date_from:
            students = filter(
                lambda student: student.payers[-1].debt_date > date_from, students)
        if date_to:
            students = filter(
                lambda student: student.payers[-1].debt_date <= date_to, students)

        return students


__all__ = ['StudentsCategory']



from .disciplines import DisciplinesCategory
from .ed_unit_students import StudentsCategory
from .ed_units import EdUnitsCategory
from .leads import LeadsCategory
from .learning_types import LearningTypesCategory
from .levels import LevelsCategory
from .locations import LocationsCategory
from .offices import OfficesCategory
from .clients import Clients

__all__ = ['StudentsCategory', 'EdUnitsCategory',
           'LeadsCategory', 'LocationsCategory', 'OfficesCategory',
           'DisciplinesCategory', 'LevelsCategory',
           'LearningTypesCategory', 'Clients']



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any

from hollihop_api_client.base import BaseCategory
from hollihop_api_client.tools import dict_to_camel, dict_to_snake

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


@dataclass
class Discipline:
    name: None | str


@dataclass
class Disciplines:
    disciplines: list[Discipline] = field(default_factory=list)


class DisciplinesCategory(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def get_disciplines(
            self
    ) -> list[Discipline]:
        data = dict_to_camel(self.handle_parameters(locals()))

        response = self.api.request(
            method='GetDisciplines',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)
        return [Discipline(_) for _ in Disciplines(**response).disciplines]


__all__ = ['DisciplinesCategory']



from dataclasses import dataclass, field
from datetime import timezone
from typing import TYPE_CHECKING, Any

from hollihop_api_client.base import BaseCategory
from hollihop_api_client.tools import dict_to_camel, dict_to_snake

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


@dataclass
class Office:
    id: None | int = None
    name: None | str = None
    location: None | str = None
    address: None | str = None
    email: None | str = None
    phone: None | str = None
    no_classrooms: None | bool = None
    time_zone: None | timezone = None
    license: None | str = None


@dataclass
class Offices:
    offices: list[Office] = field(default_factory=list)


class OfficesCategory(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def get_offices(
            self,
            id: None | int = None,
            location_id: None | int = None,
            name: None | str = None,
            license: None | str = None
    ) -> list[Office]:
        data = dict_to_camel(self.handle_parameters(locals()))

        response = self.api.request(
            method='GetOffices',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)

        return [Office(**_) for _ in Offices(**response).offices]

    def get_all_offices_name(self) -> list[str]:
        offices = self.get_offices()
        return [office.name for office in offices]

    def get_all_offices_id(self) -> list[int]:
        offices = self.get_offices()
        return [office.id for office in offices]


__all__ = ['OfficesCategory']



def test_ed_unit_students(hh_client, test_data):
    response = hh_client.ed_unit_students.get_students(
        office)
    print(response)
    assert response[-1].student_name == test_data.test_student_name


def test_ed_unit_students_debt_date(hh_client, test_data):
    response = hh_client.ed_unit_students.get_students(
        ed_unit_id=test_data.test_group_id,
        query_payers=True
    )
    assert response[-1].payers[-1].debt_date == test_data.test_debt_date



def test_get_ed_unit_by_id(hh_client, test_data):
    response = hh_client.ed_units.get_ed_units(id=test_data.test_group_id)
    assert response[-1].name == test_data.test_group_name


def test_get_ed_unit_by_teacher_id(hh_client, test_data):
    response = hh_client.ed_units.get_ed_units(
        teacher_id=test_data.test_teacher_id)
    assert response[-1].name == test_data.test_group_name


def test_get_ed_units_by_location_id(hh_client, test_data):
    response = hh_client.ed_units.get_ed_units(
        location_id=test_data.test_city_id)
    print(response)
    ed_units = list(filter(lambda ed_unit: ed_unit.name ==
                    test_data.test_group_name, response))
    assert ed_units[-1].name == test_data.test_group_name


def test_get_ed_units_by_office_id(hh_client, test_data):
    response = hh_client.ed_units.get_ed_units(
        office_or_company_id=test_data.test_office_id)
    ed_units = list(filter(lambda ed_unit: ed_unit.name ==
                    test_data.test_group_name, response))
    assert ed_units[-1].name == test_data.test_group_name



def test_get_offices_by_location_id(hh_client, test_data):
    response = hh_client.offices.get_offices(
        location_id=test_data.test_city_id)
    for office in response:
        assert office.location == test_data.test_city_name


def test_get_offices_by_location_name(hh_client, test_data):
    response = hh_client.offices.get_offices(name='Екатеринбург')
    print(response)
    for office in response:
        assert office.location == test_data.test_city_name


def test_get_office_by_id(hh_client, test_data):
    response = hh_client.offices.get_offices(id=test_data.test_office_id)
    assert response[-1].name == test_data.test_office_name



def test_ed_unit_students(hh_client, test_data):
    from pprint import pprint
    pprint(hh_client.ed_units.get_ed_unit_students(student_client_id=25759))
    pprint(hh_client.ed_units.get_ed_units(id=4536))



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any

from hollihop_api_client.base import BaseCategory
from hollihop_api_client.tools import dict_to_camel, dict_to_snake

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


@dataclass
class Level:
    name: None | str
    disciplines: None | list[str]


@dataclass
class Levels:
    levels: list[Level] = field(default_factory=list)


class LevelsCategory(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def get_levels(
            self
    ) -> list[Level]:
        data = dict_to_camel(self.handle_parameters(locals()))

        response = self.api.request(
            method='GetLevels',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)
        return [Level(**_) for _ in Levels(**response).levels]


__all__ = ['LevelsCategory']



from typing import TYPE_CHECKING

from hollihop_api_client.methods import (DisciplinesCategory, EdUnitsCategory,
                                         LeadsCategory, LearningTypesCategory,
                                         LevelsCategory, LocationsCategory,
                                         OfficesCategory, StudentsCategory, Clients)

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


class APICategories:
    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    @property
    def locations(self) -> LocationsCategory:
        return LocationsCategory(self.api)

    @property
    def offices(self) -> OfficesCategory:
        return OfficesCategory(self.api)

    @property
    def ed_units(self) -> EdUnitsCategory:
        return EdUnitsCategory(self.api)

    @property
    def ed_unit_students(self) -> StudentsCategory:
        return StudentsCategory(self.api)
    
    @property
    def clients(self) -> Clients:
        return Clients(self.api)

    @property
    def leads(self) -> LeadsCategory:
        return LeadsCategory(self.api)

    @property
    def disciplines(self) -> DisciplinesCategory:
        return DisciplinesCategory(self.api)

    @property
    def levels(self) -> LevelsCategory:
        return LevelsCategory(self.api)

    @property
    def learning_types(self) -> LearningTypesCategory:
        return LearningTypesCategory(self.api)


__all__ = ['APICategories']



from typing import TYPE_CHECKING, NoReturn

from requests import Response, Session
from requests.auth import HTTPBasicAuth

from .abc import AbstractAPI

import json
from urllib.parse import urlencode, quote_plus



class HolliHopAPIError(Exception):
    _response_not_ok = '''Запрос к {url} не выполнен.
    Статус код {status_code}.
    {error_message}'''
    _other_errors = '''{error_message}'''

    def __init__(
            self,
            error_message: str,
            status_code: None | int = None,
            url: None | str = None
    ):
        self._status_code = status_code
        self._error_message = error_message
        self._url = url

    def __str__(self) -> str:
        if self._status_code is None:
            return self._other_errors.format(
                error_message=self._error_message
            )
        else:
            return self._response_not_ok.format(
                url=self._url,
                status_code=self._status_code,
                error_message=self._error_message
            )


class HolliHopAPI(AbstractAPI):
    __api_version__ = 'Api/V2/'

    def __init__(
            self,
            domain: str | None = None,
            api_key: str | None = None
    ):
        if api_key is None:
            raise HolliHopAPIError(
                error_message='Не указан ключ доступа к API'
            )
        if domain is None:
            raise HolliHopAPIError(
                error_message='Не указан домен для доступа к API'
            )
        self._domain = domain
        self._api_key = api_key

        super().__init__(self)

    def request(
            self,
            method: str,
            http_method: str = 'GET',
            headers: dict | None = None,
            data: dict | None = None
    ) -> dict:
        
        url = self._domain + self.__api_version__ + method

        data.update({'authkey': self._api_key})

        with Session() as session:
            response = session.request(
                method=http_method,
                url=url,
                data=data,
                headers=headers
            )

        response = self._validate_response(response)

        return response
    
    def json_request(
            self, 
            method, 
            http_method: str = 'GET', 
            data: dict | list[dict] | None = None,
            headers: dict | None = None,
            ) -> dict | None:
        
        url = self._domain + self.__api_version__ + method

        with Session() as session:
            response = session.request(
                method=http_method,
                url=url + '/?authkey=' + quote_plus(f'{self._api_key}'),
                json=data,
                headers=headers
            )

        response = self._validate_response(response)

        return response

    def _validate_response(
            self,
            response: Response,
    ) -> dict | NoReturn:
        if response.status_code == 200:
            return response.json()
        else:
            from pprint import pprint
            pprint(response.request.__dict__)
            pprint(response.__dict__)
            raise HolliHopAPIError(
                response.url,
                response.status_code,
                'Ошибка выполнения запроса'
            )


__all__ = ['HolliHopAPI', 'HolliHopAPIError']



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any

from hollihop_api_client.base import BaseCategory
from hollihop_api_client.tools import dict_to_camel, dict_to_snake

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


@dataclass
class Location:
    id: None | int
    name: None | str


@dataclass
class Locations:
    locations: list[Location] = field(default_factory=list)


class LocationsCategory(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def get_locations(
            self,
            id: None | int = None,
            name: None | str = None
    ) -> list[Location]:
        data = dict_to_camel(self.handle_parameters(locals()))

        response = self.api.request(
            method='GetLocations',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)

        return [Location(**_) for _ in Locations(**response).locations]

    def get_all_locations_name(self) -> list[str]:
        locations = self.get_locations()
        return [location.name for location in locations]

    def get_all_locations_id(self) -> list[int]:
        locations = self.get_locations()
        return [location.id for location in locations]


__all__ = ['LocationsCategory']



from dataclasses import dataclass, field, asdict
from datetime import datetime, time
from typing import TYPE_CHECKING

from hollihop_api_client.base import BaseCategory
from hollihop_api_client.tools import dict_to_camel, dict_to_snake

if TYPE_CHECKING:
    from hollihop_api_client.api import AbstractAPI


@dataclass
class ScheduleItem:
    begin_date: None | datetime = None
    begin_time: None | str = None
    classroom_id: None | int = None
    classroom_name: None | str = None
    classroom_link: None | str = None
    end_date: None | datetime = None
    end_time: None | str = None
    id: None | int = None
    teacher: None | str = None
    teacher_genders: None | list[str] = None
    teacher_id: None | int = None
    teacher_ids: None | list[int] = None
    teacher_photo_urls: None | list[str] = None
    teachers: None | list[str] = None
    weekdays: None | int = None

    def __post_init__(self):
        if not self.begin_time is None:
            self.begin_date = datetime.fromisoformat(self.begin_date)
        if not self.end_date is None:
            self.end_date = datetime.fromisoformat(self.end_date)


@dataclass
class Day:
    date: None | datetime = None
    minutes: None | float = None
    pass_: None | bool = field(init=False)
    Pass: None | bool = None
    description: None | str = None
    student_payable_minutes: None | float = None
    teacher_payable_minutes: None | float = None
    accepted: bool | None = None
    accepted_description: str | None = None

    def __post_init__(self):
        if not self.date is None:
            self.date = datetime.fromisoformat(self.date)
        self.pass_ = self.Pass
        del self.Pass
        # delattr(self, 'Pass')


@dataclass
class FiscalInfo:
    price_id: None | int = None
    price_name: None | str = None
    price_value: None | str = None
    units: None | str = None
    units28: None | str = None
    units7: None | str = None
    value: None | str = None
    value28: None | str = None
    value7: None | str = None

    def __post_init__(self):
        if not self.price_name is None:
            self.price_name = self.price_name.replace('\xa0', ' ')
        if not self.price_value is None:
            self.price_value = self.price_value.replace('\xa0', ' ')
        if not self.value is None:
            self.value = self.value.replace('\xa0', ' ')
        if not self.value28 is None:
            self.value28 = self.value28.replace('\xa0', ' ')
        if not self.value7 is None:
            self.value7 = self.value7.replace('\xa0', ' ')


@dataclass
class PriceValue:
    students: None | int = None
    value: None | str = None
    value_currency: None | str = None
    value_quantity: None | float = None

    def __post_init__(self):
        if not self.value is None:
            self.value = self.value.replace('\xa0', ' ')


@dataclass
class TeacherPrice:
    price_id: None | int = None
    price_name: None | str = None
    price_units: None | str = None
    price_units_quantity: None | int = None
    price_units_type: None | str = None
    price_values: None | list[PriceValue] = None
    teacher_id: None | int = None

    def __post_init__(self):
        if not self.price_values is None:
            self.price_values = [PriceValue(**_) for _ in self.price_values]


@dataclass
class EdUnit:
    id: int | None = None
    type: str | None = None
    name: str | None = None
    corporative: bool = False
    office_or_company_id: int | None = None
    office_or_company_name: str | None = None
    office_or_company_address: str | None = None
    office_time_zone: time | None = None
    discipline: str | None = None
    level: str | None = None
    maturity: str | None = None
    learning_type: str | None = None
    extra_fields: list | None = None
    students_count: int | None = None
    vacancies: int | None = None
    study_units_in_range: str | None = None
    description: str | None = None
    company_contract_number: str | None = None
    company_contract_date: datetime | None = None
    schedule_items: list | None = None
    days: list[Day] | None = None
    fiscal_info: FiscalInfo | list = None
    teacher_prices: None | list[TeacherPrice] = None
    price_values: None | list = None
    assignee: None | dict = None

    def __post_init__(self):
        if not self.schedule_items is None:
            self.schedule_items = [ScheduleItem(
                **_) for _ in self.schedule_items]
        if not self.days is None:
            self.days = [Day(**_) for _ in self.days]
        if not self.fiscal_info is None:
            self.fiscal_info = FiscalInfo(**self.fiscal_info)
        if not self.teacher_prices is None:
            self.teacher_prices = [TeacherPrice(
                **_) for _ in self.teacher_prices]

    def __eq__(self, other: object):
        if not isinstance(other, EdUnit):
            return NotImplemented
        return self.id == other.id

    def __hash__(self):
        return hash(self.id)


@dataclass(frozen=True)
class Statuses:
    Reserve = 'Reserve'
    Forming = 'Forming'
    Working = 'Working'
    Stopped = 'Stopped'
    Finished = 'Finished'


@dataclass
class EdUnits:
    ed_units: list[EdUnit] = field(default_factory=list)
    now: datetime = field(default_factory=datetime.fromtimestamp)


def array_to_one_str(data: dict):
    for key in data.keys():
        if type(data[key]) == list:
            data[key] = ",".join(data[key])
    return data


@dataclass
class SetStudentPasses: 
    date: str | None = None
    ed_unit_id: int | None = None
    student_client_id: int | None = None
    pass_: bool | None = None
    payable: bool | None = None
    description: str | None = None
    acceptedDescription: str | None = None
    overwriteAcceptedManually: bool | None = None


@dataclass
class AddEdUnitStudent:
    ed_unit_id: int | None = None
    student_client_id: int | None = None
    status: str | None = None
    begin: str | None = None
    end: str | None = None


class EdUnitsCategory(BaseCategory):

    def __init__(self, api: 'AbstractAPI'):
        self.api = api

    def get_ed_units(
            self,
            id: None | int = None,
            types: None | str = None,

            date_from: None | str = None,
            date_to: None | str = None,

            statuses: None | str = None,
            office_or_company_id: None | str = None,
            location_id: None | list[int] = None,
            disciplines: None | list[str] = None,
            levels: None | list[str] = None,
            maturities: None | str = None,
            corporative: None | bool = None,
            learning_types: None | list[str] = None,
            teacher_id: None | int = None,
            query_days: None | bool = None,
            query_fiscal_info: None | bool = None,
            query_teacher_prices: None | bool = None,
    ) -> list[EdUnit]:
        data = dict_to_camel(self.handle_parameters(locals()))
        data = array_to_one_str(data)
        response = self.api.request(
            method='GetEdUnits',
            http_method='GET',
            data=data
        )

        response = dict_to_snake(response)

        return [EdUnit(**_) for _ in EdUnits(**response).ed_units]
    
    def create_student_pass_data(
            self,
            date: str | None = None,
            ed_unit_id: int | None = None,
            student_client_id: int | None = None,
            pass_: bool | None = None,
            payable: bool | None = None,
            description: str | None = None,
            acceptedDescription: str | None = None,
            overwriteAcceptedManually: bool | None = True
            ) -> dict:
        return self.handle_parameters(locals())
    
    def set_student_passes(
            self,
            data: list[SetStudentPasses]
            ):
        params = self.handle_parameters(locals())
        
        request_data = []
        for _ in params['data']:
            if 'pass_' in _:
                pass_ = _.pop('pass_')
                _ = {**_, 'pass': pass_}
            request_data.append(dict_to_camel(_))

        response = self.api.json_request(
            method='SetStudentPasses',
            http_method='POST',
            data=request_data,
            headers={'Content-Type':'application/json'}
        )

        response = dict_to_snake(response)

        return response
    

    def add_ed_unit_student(
            self,
            ed_unit_id: int | None = None,
            student_client_id: int | None = None,
            status: str | None = None,
            begin: str | None = None,
            end: str | None = None,
            ):
        request_data = self.handle_parameters(locals())
        request_data = dict_to_camel(request_data)

        response = self.api.json_request(
            method='AddEdUnitStudent',
            http_method='POST',
            data=request_data,
            headers={'Content-Type':'application/json'}
        )

        response = dict_to_snake(response)

        return response
        


    def get_all_ed_units_name(self, **kwargs) -> list[str]:
        ed_units = self.get_ed_units(**kwargs)
        return [ed_unit.name for ed_unit in ed_units]

    def get_all_ed_units_id(self, **kwargs) -> list[int]:
        ed_units = self.get_ed_units(**kwargs)
        return [ed_unit.id for ed_unit in ed_units]


__all__ = ['EdUnitsCategory']
